import shutil
import tempfile

import click

from datahub import __package_name__
from datahub.cli.cli_utils import get_url_and_token
from datahub.cli.json_file import check_mce_file
from datahub.graph_consistency import check_data_platform
from datahub.ingestion.run.pipeline import Pipeline
from datahub.ingestion.sink.sink_registry import sink_registry
from datahub.ingestion.source.source_registry import source_registry
from datahub.ingestion.transformer.transform_registry import transform_registry


@click.group()
def check() -> None:
    """Helper commands for checking various aspects of DataHub."""
    pass


@check.command()
@click.argument("json-file", type=click.Path(exists=True, dir_okay=False))
@click.option(
    "--rewrite",
    default=False,
    is_flag=True,
    help="Rewrite the JSON file to it's canonical form.",
)
@click.option(
    "--unpack-mces", default=False, is_flag=True, help="Converts MCEs into MCPs"
)
def metadata_file(json_file: str, rewrite: bool, unpack_mces: bool) -> None:
    """Check the schema of a metadata (MCE or MCP) JSON file."""

    if not rewrite:
        report = check_mce_file(json_file)
        click.echo(report)

    else:
        with tempfile.NamedTemporaryFile() as out_file:
            pipeline = Pipeline.create(
                {
                    "source": {
                        "type": "file",
                        "config": {"filename": json_file},
                        "extractor": "generic",
                        "extractor_config": {
                            "set_system_metadata": False,
                            "unpack_mces_into_mcps": unpack_mces,
                        },
                    },
                    "sink": {
                        "type": "file",
                        "config": {"filename": out_file.name},
                    },
                }
            )

            pipeline.run()
            pipeline.raise_from_status()

            shutil.copy(out_file.name, json_file)


@check.command()
@click.option(
    "--verbose",
    "-v",
    type=bool,
    is_flag=True,
    default=False,
    help="Include extra information for each plugin.",
)
def plugins(verbose: bool) -> None:
    """List the enabled ingestion plugins."""

    click.secho("Sources:", bold=True)
    click.echo(source_registry.summary(verbose=verbose))
    click.echo()
    click.secho("Sinks:", bold=True)
    click.echo(sink_registry.summary(verbose=verbose))
    click.echo()
    click.secho("Transformers:", bold=True)
    click.echo(transform_registry.summary(verbose=verbose, col_width=30))
    click.echo()
    if not verbose:
        click.echo("For details on why a plugin is disabled, rerun with '--verbose'")
    click.echo(
        f"If a plugin is disabled, try running: pip install '{__package_name__}[<plugin>]'"
    )


@check.command
def graph_consistency() -> None:
    gms_endpoint, gms_token = get_url_and_token()
    check_data_platform.check(gms_endpoint, gms_token)



# flake8: noqa

# This file is autogenerated by /metadata-ingestion/scripts/avro_codegen.py
# Do not modify manually!

# pylint: skip-file
# fmt: off
from .....schema_classes import CorpGroupEditableInfoClass
from .....schema_classes import CorpGroupInfoClass
from .....schema_classes import CorpUserAppearanceSettingsClass
from .....schema_classes import CorpUserCredentialsClass
from .....schema_classes import CorpUserEditableInfoClass
from .....schema_classes import CorpUserInfoClass
from .....schema_classes import CorpUserSettingsClass
from .....schema_classes import CorpUserStatusClass
from .....schema_classes import GroupMembershipClass
from .....schema_classes import InviteTokenClass
from .....schema_classes import NativeGroupMembershipClass
from .....schema_classes import RoleMembershipClass


CorpGroupEditableInfo = CorpGroupEditableInfoClass
CorpGroupInfo = CorpGroupInfoClass
CorpUserAppearanceSettings = CorpUserAppearanceSettingsClass
CorpUserCredentials = CorpUserCredentialsClass
CorpUserEditableInfo = CorpUserEditableInfoClass
CorpUserInfo = CorpUserInfoClass
CorpUserSettings = CorpUserSettingsClass
CorpUserStatus = CorpUserStatusClass
GroupMembership = GroupMembershipClass
InviteToken = InviteTokenClass
NativeGroupMembership = NativeGroupMembershipClass
RoleMembership = RoleMembershipClass

# fmt: on



from dataclasses import dataclass, field
from typing import Dict

from datahub.ingestion.source.sql.sql_common import SQLSourceReport
from datahub.utilities.stats_collections import TopKDict


@dataclass
class UnityCatalogReport(SQLSourceReport):
    scanned_metastore: int = 0
    scanned_catalog: int = 0
    scanned_schema: int = 0
    scanned_table: int = 0
    num_catalogs_to_scan: Dict[str, int] = field(default_factory=TopKDict)
    num_schemas_to_scan: Dict[str, int] = field(default_factory=TopKDict)
    num_tables_to_scan: Dict[str, int] = field(default_factory=TopKDict)

    def increment_scanned_metastore(self, count: int = 1) -> None:
        self.scanned_metastore = self.scanned_metastore + count

    def increment_scanned_catalog(self, count: int = 1) -> None:
        self.scanned_catalog = self.scanned_catalog + count

    def increment_scanned_schema(self, count: int = 1) -> None:
        self.scanned_schema = self.scanned_schema + count

    def increment_scanned_table(self, count: int = 1) -> None:
        self.scanned_table = self.scanned_table + count



import json
from collections import defaultdict
from dataclasses import dataclass
from typing import Any, Dict, Iterable, List, Optional

from datahub.emitter.serialization_helper import pre_json_transform
from datahub.metadata.schema_classes import (
    ChangeTypeClass,
    GenericAspectClass,
    KafkaAuditHeaderClass,
    MetadataChangeProposalClass,
    SystemMetadataClass,
)


def _recursive_to_obj(obj: Any) -> Any:
    if isinstance(obj, list):
        return [_recursive_to_obj(v) for v in obj]
    elif hasattr(obj, "to_obj"):
        return obj.to_obj()
    else:
        return obj


@dataclass
class _Patch:
    op: str  # one of ['add', 'remove', 'replace']; we don't support move, copy or test
    path: str
    value: Any

    def to_obj(self) -> Dict:
        return {
            "op": self.op,
            "path": self.path,
            "value": _recursive_to_obj(self.value),
        }


class MetadataPatchProposal:
    urn: str
    entity_type: str

    # mapping: aspectName -> list of patches
    patches: Dict[str, List[_Patch]]

    def __init__(
        self,
        urn: str,
        entity_type: str,
        system_metadata: Optional[SystemMetadataClass] = None,
        audit_header: Optional[KafkaAuditHeaderClass] = None,
    ) -> None:
        self.urn = urn
        self.entity_type = entity_type
        self.system_metadata = system_metadata
        self.audit_header = audit_header
        self.patches = defaultdict(list)

    def _add_patch(self, aspect_name: str, op: str, path: str, value: Any) -> None:
        # TODO: Validate that aspectName is a valid aspect for this entityType
        self.patches[aspect_name].append(_Patch(op, path, value))

    def build(self) -> Iterable[MetadataChangeProposalClass]:
        return [
            MetadataChangeProposalClass(
                entityUrn=self.urn,
                entityType=self.entity_type,
                changeType=ChangeTypeClass.PATCH,
                aspectName=aspect_name,
                aspect=GenericAspectClass(
                    value=json.dumps(
                        pre_json_transform(_recursive_to_obj(patches))
                    ).encode(),
                    contentType="application/json-patch+json",
                ),
                auditHeader=self.audit_header,
                systemMetadata=self.system_metadata,
            )
            for aspect_name, patches in self.patches.items()
        ]
